package	jp.co.sds_corp.ratemixer.fx;

import java.math.BigDecimal;
import java.util.Properties;

import jp.co.sds_corp.fxutil.CcyPair;
import jp.co.sds_corp.fxutil.ForexRate;
import jp.co.sds_corp.transfer.IData;
import jp.co.sds_corp.transfer.IDataKey;
import jp.co.sds_corp.util.common.CmnDate;

/**
* 為替レート情報を保持するクラスです。
*/
public class FxRate implements IData
{
	private static final long serialVersionUID = 1L;
	FxRateKey	key;
	BigDecimal	bid;
	BigDecimal	ask;
	String		strTime;
	CmnDate		recvTime;
	CmnDate		updateTime;
	CmnDate		valueDate;
	boolean		init;
	boolean		valid;
	Properties	prop = new Properties();

	/**
	* 指定された通貨ペアと期間で、レート情報を構築します。
	* @param	key		為替レートキー
	*/
	public FxRate( FxRateKey key ) {
		this.key = key;
		this.updateTime = new CmnDate();
	}

	/**
	* 指定された通貨ペアと期間で、レート情報を構築します。
	* @param	pair	通貨ペア
	* @param	term	期間
	*/
	public FxRate( CcyPair pair, String term ) {
		this( new FxRateKey( pair, term ));
	}

	/**
	* コピーコンストラクタです。
	* @param	obj		コピー元のレート情報です。
	*/
	public FxRate( FxRate obj ) {
		this( (FxRateKey )obj.getKey() );
	}

	/**
	* 引数で指定されたオブジェクトとレートを比較します。
	* @param	o		比較元のレート情報
	* @return	比較結果。変更無し:true 変更有り:false
	*/
	public boolean equals( Object o ) {
		if ( o == null || !( o instanceof FxRate )) {
			return false;
		}
		IData cmp = (IData )o;
		if ( !getKey().equals( cmp.getKey() )) {
			return false;
		}
		FxRate rc = (FxRate )cmp;
		if (( bid == null && rc.bid != null ) ||
			( ask == null && rc.ask != null ) ||
			( bid != null && rc.bid == null ) ||
			( ask != null && rc.ask == null ) )
		{
			return false;
		}

		if ( bid != null && rc.bid != null && bid.compareTo( rc.bid ) != 0 ) {
			return false;
		}
		if ( ask != null && rc.ask != null && ask.compareTo( rc.ask ) != 0 ) {
			return false;
		}
		return true;
	}

	public int hashCode() {
		return getKey().hashCode();
	}

	/**
	* レートの文字列表現を返します。
	* @return	レートの文字列表現
	*/
	public String toString() {
		String ret =  getCcyPair() +":"+ getTerm() +":"+ bid +":"+ ask;
		if ( prop.size() > 0 ) {
			ret += ":" + prop.toString();
		}
		ret += ":" + updateTime.format( "yyyy/MM/dd HH:mm:ss" );
		return ret;
	}

	/**
	* 指定されたキーと値を設定します。
	* @param	key		キー
	* @param	value	値
	* @return	以前に設定されている値。無ければnull
	*/
	public Object setProperty( String key, String value ) {
		Object ret = prop.setProperty( key, value );
		return ret;
	}
	/**
	* 指定されたキーを持つプロパティを、プロパティリストから探します。
	* @param	key		キー
	* @return	設定されている値
	*/
	public String getProperty( String key ) {
		String ret = prop.getProperty( key );
		return ret;
	}
	/**
	* 現在設定されているプロパティを取得します。
	* @return	設定されているプロパティ
	*/
	public Properties getProperties() {
		return prop;
	}

	public void copyTo( ForexRate var ) {
		var.setBid( getBid() );
		var.setAsk( getAsk() );
		var.getProperties().clear();
		var.getProperties().putAll( getProperties() );
	}
	public void copyFrom( ForexRate var ) {
		setBid( var.getBid() );
		setAsk( var.getAsk() );
		getProperties().clear();
		getProperties().putAll( var.getProperties() );
	}

	/**
	* キー(key)を設定します。
	* @param	var		設定するデータ
	*/
	public void setKey( IDataKey var ) {
		this.key = (FxRateKey )var;
	}
	/**
	* キー(key)を取得します。
	* @return	keyの内容
	*/
	public IDataKey getKey() {
		return key;
	}

	/**
	* 通貨ペア(pair)を取得します。
	* @return	pairの内容
	*/
	public CcyPair getCcyPair() {
		return key.getCcyPair();
	}

	/**
	* 期間コード(term)を取得します。
	* @return	termの内容
	*/
	public String getTerm() {
		return key.getTerm();
	}

	/**
	* Bidレート(bid)を設定します。
	* @param	var		設定するデータ
	*/
	public void setBid( BigDecimal var ) {
		if ( var == null ) {
			bid = null;
			return;
		}
		int point;
		if ( getTerm().equals( "SP" )) {
			point = getCcyPair().getPoint();
		} else {
			point = getCcyPair().getFwdPoint();
		}
		this.bid = var.setScale( point, BigDecimal.ROUND_DOWN );
	}
	/**
	* Bidレート(bid)を取得します。
	* @return	bidの内容
	*/
	public BigDecimal getBid() {
		return bid;
	}

	/**
	* Askレート(ask)を設定します。
	* @param	var		設定するデータ
	*/
	public void setAsk( BigDecimal var ) {
		if ( var == null ) {
			ask = null;
			return;
		}
		int point;
		if ( getTerm().equals( "SP" )) {
			point = getCcyPair().getPoint();
		} else {
			point = getCcyPair().getFwdPoint();
		}
		this.ask = var.setScale( point, BigDecimal.ROUND_DOWN );
	}
	/**
	* Askレート(ask)を取得します。
	* @return	askの内容
	*/
	public BigDecimal getAsk() {
		return ask;
	}

	/**
	* 受信時刻(updateTime)を設定します。
	* @param	var		設定するデータ
	*/
	public void setUpdateTime( CmnDate var ) {
		this.updateTime = var ;
	}
	/**
	* 受信時刻(updateTime)を取得します。
	* @return	updateTimeの内容
	*/
	public CmnDate getUpdateTime() {
		return updateTime;
	}

	/**
	* 受信時刻(recvTime)を設定します。
	* @param	var		設定するデータ
	*/
	public void setRecvTime( CmnDate var ) {
		this.recvTime = var ;
	}
	/**
	* 受信時刻(recvTime)を取得します。
	* @return	recvTimeの内容
	*/
	public CmnDate getRecvTime() {
		return recvTime;
	}

	/**
	* 初期化済フラグ(init)を設定します。
	* @param	var		設定するデータ
	*/
	public void setInit( boolean var ) {
		this.init = var ;
	}
	/**
	* 初期化済フラグ(init)を取得します。
	* @return	initの内容
	*/
	public boolean isInit() {
		return init;
	}

	/**
	* 有効フラグ(valid)を設定します。
	* @param	var		設定するデータ
	*/
	public void setValid( boolean var ) {
		this.valid = var ;
	}
	/**
	* 有効フラグ(valid)を取得します。
	* @return	validの内容
	*/
	public boolean isValid() {
		return valid;
	}

	/**
	* Value日(valueDate)を設定します。
	* @param	var		設定するデータ
	*/
	public void setValueDate( String var ) {
		try {
			this.valueDate = new CmnDate( var );
		} catch ( Exception ex ) {
		}
	}
	/**
	* Value日(valueDate)を設定します。
	* @param	var		設定するデータ
	*/
	public void setValueDate( CmnDate var ) {
		this.valueDate = var;
	}
	/**
	* Value日(valueDate)を取得します。
	* @return	valueDateの内容
	*/
	public CmnDate getValueDate() {
		return valueDate;
	}

}
